%{
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include "parse.h"

#define DEBUG 1
#ifdef DEBUG
#define TRACE printf("parser: reduce at line %d\n", __LINE__);
#else
#define TRACE
#endif
%}

%token <iValue> vABS vAND vASC vATN vAUTO vCALL vCHAR vCHR vCIRCLE vCLEAR vCLOAD vCLS vCONT
%token <iValue> vCOS vCSAVE vCURMOV vCURSET vDATA vDEEK vDEF vDIM vDOKE vDRAW vEDIT vELSE
%token <iValue> vEND vEXP vEXPLODE vFALSE vFILL vFN vFOR vFRE vGET vGOSUB vGOTO vGRAB
%token <iValue> vHEX vHIMEM vHIRES vIF vINK vINPUT vINT vKEY vLEFT vLEN vLET vLIST vLLIST
%token <iValue> vLN vLOG vLORES vLPRINT vMID vMUSIC vNEW vNEXT vNOT vON vOR vPAPER vPATTERN
%token <iValue> vPEEK vPI vPING vPLAY vPLOT vPOINT vPOKE vPOP vPOS vPRINT vPULL vREAD vRECALL
%token <iValue> vRELEASE vREM vREPEAT vRESTORE vRETURN vRIGHT vRND vRUN vSCRN vSGN vSHOOT
%token <iValue> vSIN vSOUND vSPC vSQR vSTEP vSTOP vSTORE vSTR vTAB vTAN vTEXT vTHEN vTO
%token <iValue> vTROFF vTRON vTRUE vUNTIL vUSR vVAL vWAIT vZAP
%token vSEPARATOR

%token <iValue> vSHRIEK vAMPERSAND
%token vAT vCR

%token <iValue> vCstINTEGER
%token <fValue> vCstFLOAT
%token <sValue> vCstSTRING
%token <sValue> vVarINTEGER
%token <sValue> vVarFLOAT
%token <sValue> vVarSTRING
%token <sValue> vHEXA
%token <sValue> vCstSTRING_NOT_DEFINED
%token <sValue> vNOT_FOUND
%token <sValue> vKEYWORD_IN_WORD
%token <sValue> vERROR_UNKNOWN

%union {
 int        iValue;  /* Integer Value */
 float      fValue;  /* Float Value   */
 char     * sValue;  /* String Value  */
 nodeType * nPtr;    /* Node pointer  */
};


%left vEQ vNE vLT vLE vGT vGE
%left vPLUS vMINUS
%left vMULTIPLY vDIVIDE
%left vAND vOR
%right vPOWER
%nonassoc IFX
/* %nonassoc vELSE */

%type <nPtr> expr command instructions ligne lignes function expr_string
%type <nPtr> cload_expr csave_expr recall_expr store_expr data_expr def_expr dim_expr dim_2_expr
%type <nPtr> for_expr if_expr gosub_expr goto_expr input_expr print_expr print_2_expr read_expr
%type <iValue> vPLUS vMINUS vMULTIPLY vDIVIDE vAND vOR vPOWER
%type <iValue> vEQ vNE vGT vGE vLT vLE
%start programme
%%

programme:
   lignes                          { printf("PRG\n"); /* printPRG(pListeLines); */ COMPILATION(pPRG); }
 ;

lignes:
    lignes ligne                   { printf("LINE\n"); addLine($2, &pListeLines); }
  | /* NULL */                     { }
 ;

ligne:
   vCstINTEGER instructions vCR    { $$ = Op(LIGNE, 2, iConst($1), $2); }
 ;

instructions:
   command                         { printf("<command>\n");             $$ = $1;                            }
 | vSEPARATOR                      { printf("<:>\n");                   $$ = Op(vSEPARATOR, 2, NULL, NULL); }
 | command vSEPARATOR              { printf("<command:>\n");            $$ = $1;                            }
 | vSEPARATOR command              { printf("<:command>\n");            $$ = $2;                            }
 | command vSEPARATOR instructions { printf("<command:instruction>\n"); $$ = Op(0  , 3, $1, Op(vSEPARATOR, 2, NULL, NULL), $3 ); }
 | if_expr                         { printf("<if>");                    $$ = $1;                            }
;

command:
   vVarFLOAT   vEQ  expr                   { $$ = Op(47     , 2, fVar($1)  , $3);                      }
 | vVarINTEGER vEQ  expr                   { $$ = Op(47     , 2, iVar($1)  , $3);                      }
 | vVarSTRING  vEQ  expr_string            { $$ = Op(47     , 2, sVar($1)  , $3);                      }
 | vVarSTRING  vEQ  vKEY                   { $$ = Op($3     , 2, sVar($1)  , NULL);                    }   
 | vCALL expr                              { $$ = Op($1     , 2, $2        , NULL);                    }
 | vCHAR expr ',' expr ',' expr            { $$ = Op($1     , 3, $2        , $4, $6);                  }
 | vCIRCLE expr ',' expr                   { $$ = Op($1     , 2, $2        , $4);                      }
 | vCLEAR                                  { $$ = Op($1     , 2, NULL      , NULL);                    }
 | cload_expr                              { $$ = $1;                                                  }
 | vCLS                                    { $$ = Op($1     , 2, NULL      , NULL);                    }
 | csave_expr                              { $$ = $1;                                                  }
 | vCURMOV expr ',' expr ',' expr          { $$ = Op($1     , 3, $2        , $4, $6);                  }
 | vCURSET expr ',' expr ',' expr          { $$ = Op($1     , 3, $2        , $4, $6);                  }
 | vDATA data_expr                         { $$ = Op($1     , 2, $2        , NULL);                    }
 | vDEF def_expr                           { $$ = Op($1     , 2, $2        , NULL);                    }
 | dim_expr                                { $$ = $1;                                                  }
 | vDOKE expr ',' expr                     { $$ = Op($1     , 2, $2        , $4);                      }
 | vDRAW expr ',' expr ',' expr            { $$ = Op($1     , 3, $2        , $4, $6);                  }
 | vEND                                    { $$ = Op($1     , 2, NULL      , NULL);                    }
 | vEXPLODE                                { $$ = Op($1     , 2, NULL      , NULL);                    }
 | vFILL expr ',' expr ',' expr            { $$ = Op($1     , 3, $2        , $4, $6);                  }
 | for_expr                                { $$ = $1;                                                  }
 | vGET vVarFLOAT                          { $$ = Op($1     , 2, fVar($2)  , NULL);                    }
 | vGET vVarINTEGER                        { $$ = Op($1     , 2, iVar($2)  , NULL);                    }
 | vGET vVarSTRING                         { $$ = Op($1     , 2, sVar($2)  , NULL);                    }
 | vGOSUB gosub_expr                       { $$ = Op($1     , 2, $2        , NULL);                    }
 | vGOTO goto_expr                         { $$ = Op($1     , 2, $2        , NULL);                    }
 | vGRAB                                   { $$ = Op($1     , 2, NULL      , NULL);                    }
 | vHIMEM expr                             { $$ = Op($1     , 2, $2        , NULL);                    }
 | vHIRES                                  { $$ = Op($1     , 2, NULL      , NULL);                    }
 | vINK expr                               { $$ = Op($1     , 2, $2        , NULL);                    }
 | vINPUT input_expr                       { $$ = Op($1     , 2, $2        , NULL);                    }
 | vINPUT vCstSTRING ',' input_expr        { $$ = Op($1     , 2, sConst($2), $4);                      }
 | vLET vVarFLOAT vEQ expr                 { $$ = Op($1     , 2, fVar($2)  , $4);                      }
 | vLET vVarINTEGER vEQ expr               { $$ = Op($1     , 2, iVar($2)  , $4);                      }
 | vLET vVarSTRING vEQ expr                { $$ = Op($1     , 2, sVar($2)  , $4);                      }
 | vLORES vCstINTEGER                      { $$ = Op($1     , 1, iConst($2), NULL);                    }
 | vMUSIC expr ',' expr ',' expr ',' expr  { $$ = Op($1     , 4, $2        , $4, $6, $8);              }
 | vNEXT vVarINTEGER                       { $$ = Op($1     , 2, iVar($2)  , NULL);                    }
 | vNEXT vVarFLOAT                         { $$ = Op($1     , 2, fVar($2)  , NULL);                    }
 | vON vVarINTEGER vGOTO goto_expr         { $$ = Op($1     , 2, iVar($2)  , Op(vGOTO , 2, $4, NULL)); }
 | vON vVarFLOAT vGOTO goto_expr           { $$ = Op($1     , 2, fVar($2)  , Op(vGOTO , 2, $4, NULL)); }
 | vON vVarINTEGER vGOSUB gosub_expr       { $$ = Op($1     , 2, iVar($2)  , Op(vGOSUB, 2, $4, NULL)); }
 | vON vVarFLOAT vGOSUB gosub_expr         { $$ = Op($1     , 2, fVar($2)  , Op(vGOSUB, 2, $4, NULL)); }
 | vPAPER expr                             { $$ = Op($1     , 2, $2        , NULL);                    }
 | vPATTERN expr                           { $$ = Op($1     , 2, $2        , NULL);                    }
 | vPING                                   { $$ = Op($1     , 2, NULL      , NULL);                    }
 | vPLAY expr ',' expr ',' expr ',' expr   { $$ = Op($1     , 4, $2        , $4, $6, $8);              }
 | vPLOT expr ',' expr ',' expr            { $$ = Op($1     , 3, $2        , $4, $6);                  }
 | vPOINT '(' expr ',' expr ')'            { $$ = Op($1     , 2, $3        , $5);                      }
 | vPOKE  expr ',' expr                    { $$ = Op($1     , 2, $2        , $4);                      }
 | vPOP                                    { $$ = Op($1     , 2, NULL      , NULL);                    }
 | vPRINT print_expr                       { $$ = Op($1     , 2, $2        , NULL);                    }
 | vPULL                                   { $$ = Op($1     , 2, NULL      , NULL);                    }
 | vREAD read_expr                         { $$ = Op($1     , 2, $2        , NULL);                    }
 | recall_expr                             { $$ = $1;                                                  }
 | vRELEASE                                { $$ = Op($1     , 2, NULL      , NULL);                    }
 | vREPEAT                                 { $$ = Op($1     , 2, NULL      , NULL);                    }
 | vRESTORE                                { $$ = Op($1     , 2, NULL      , NULL);                    }
 | vRETURN                                 { $$ = Op($1     , 2, NULL      , NULL);                    }
 | vSHOOT                                  { $$ = Op($1     , 2, NULL      , NULL);                    }
 | vSOUND expr ',' expr ',' expr           { $$ = Op($1     , 3, $2        , $4, $6);                  }
 | vSTOP                                   { $$ = Op($1     , 2, NULL      , NULL);                    }
 | store_expr                              { $$ = $1;                                                  }
 | vTEXT                                   { $$ = Op($1     , 2, NULL      , NULL);                    }
 | vTRUE                                   { $$ = Op($1     , 2, NULL      , NULL);                    }
 | vUNTIL expr                             { $$ = Op($1     , 2, $2        , NULL);                    }
 | vWAIT expr                              { $$ = Op($1     , 2, $2        , NULL);                    }
 | vZAP                                    { $$ = Op($1     , 2, NULL      , NULL);                    }
 | vSHRIEK                                 { $$ = Op($1     , 2, NULL      , NULL);                    }
 | vCONT       { printf("parser: ERREUR : CONT ne peut etre utilise dans un programme compile\n");     }
 | vEDIT       { printf("parser: ERREUR : EDIT ne peut etre utilise dans un programme compile\n");     }
 | vELSE       { printf("parser: ERREUR : ELSE est associe avec IF ... THEN ... [ELSE]\n");            }
 | vLIST       { printf("parser: ERREUR : LIST ne peut etre utilise dans un programme compile\n");     }
 | vLLIST      { printf("parser: ERREUR : LLIST ne peut etre utilise dans un programme compile\n");    }
 | vLPRINT     { printf("parser: ERREUR : LPRINT ne peut etre utilise dans un programme compile\n");   }
 | vNEW        { printf("parser: ERREUR : NEW ne peut etre utilise dans un programme compile\n");      }
 | vREM        { printf("parser:        : REM <%s> ", yylval.sValue);                                  }
 | vRUN        { printf("parser: ERREUR : RUN ne peut etre utilise dans un programme compile\n");      }
 | vSTEP       { printf("parser: ERREUR : STEP est associe avec FOR ... TO ... STEP ...\n");           }
 | vTHEN       { printf("parser: ERREUR : THEN est associe avec IF ... THEN ... [ELSE]\n");            }
 | vTO         { printf("parser: ERREUR : TO est associe avec FOR ... TO ... STEP\n");                 }
 | vTROFF      { printf("parser: ERREUR : TROFF ne peut etre utilise dans un programme compile\n");    }
 | vTRON       { printf("parser: ERREUR : TRON ne peut etre utilise dans un programme compile\n");     }
 | error       { printf("parser: ERREUR : COMMANDE INCONNUE\n");                                       }
 ;

function:
   vABS '(' expr ')'                              { $$ = Op($1, 2, $3        , NULL);  }
 | vASC '(' expr_string ')'                       { $$ = Op($1, 2, $3        , NULL);  } 
 | vATN '(' expr ')'                              { $$ = Op($1, 2, $3        , NULL);  }
 | vCOS '(' expr ')'                              { $$ = Op($1, 2, $3        , NULL);  }
 | vDEEK '(' expr ')'                             { $$ = Op($1, 2, $3        , NULL);  }
 | vEXP '(' expr ')'                              { $$ = Op($1, 2, $3        , NULL);  }
 | vFALSE                                         { $$ = Op($1, 0, NULL      , NULL);  }
 | vFN  vVarFLOAT '(' expr ')'                    { $$ = Op($1, 2, sVar($2)  , $4);    }
 | vFRE '(' '0' ')'                               { $$ = Op($1, 2, iConst(0) , NULL);  }
 | vFRE vCstSTRING                                { $$ = Op($1, 2, sConst($2), NULL);  }
 | vHEX '(' expr ')'                              { $$ = Op($1, 2, $3        , NULL);  }
 | vINT '(' expr ')'                              { $$ = Op($1, 2, $3        , NULL);  }
 | vLEN '(' expr_string ')'                       { $$ = Op($1, 2, $3        , NULL);  }
 | vLN '(' expr ')'                               { $$ = Op($1, 2, $3        , NULL);  }
 | vLOG '(' expr ')'                              { $$ = Op($1, 2, $3        , NULL);  }
 | vNOT '(' expr ')'                              { $$ = Op($1, 2, $3        , NULL);  }
 | vPEEK '(' expr ')'                             { $$ = Op($1, 2, $3        , NULL);  }
 | vPI                                            { $$ = fConst(PI);                   }
 | vPOS '(' '0' ')'                               { $$ = Op($1, 2, iConst(0) , NULL);  }
 | vPOS '(' '1' ')'                               { $$ = Op($1, 2, iConst(1) , NULL);  }
 | vRND '(' expr ')'                              { $$ = Op($1, 2, $3        , NULL);  }
 | vSCRN '(' expr ',' expr ')'                    { $$ = Op($1, 2, $3        , $5);    }
 | vSGN  '(' expr ')'                             { $$ = Op($1, 2, $3        , NULL);  }
 | vSIN '(' expr ')'                              { $$ = Op($1, 2, $3        , NULL);  }
 | vTRUE                                          { $$ = Op($1, 0, NULL      , NULL);  }
 | vUSR '(' expr ')'                              { $$ = Op($1, 2, $3        , NULL);  }
 | vVAL '(' expr_string ')'                       { $$ = Op($1, 2, $3        , NULL);  }
 | vSPC '(' expr ')'                              { $$ = Op($1, 2, $3        , NULL);  }
 | vSQR '(' expr ')'                              { $$ = Op($1, 2, $3        , NULL);  }
 | vSTR '(' expr ')'                              { $$ = Op($1, 2, $3        , NULL);  }
 | vTAB '(' expr ')'                              { $$ = Op($1, 2, $3        , NULL);  }
 | vTAN '(' expr ')'                              { $$ = Op($1, 2, $3        , NULL);  }
 | vAMPERSAND                                     { $$ = Op($1, 2, NULL      , NULL);  }
 | expr vAND expr                                 { $$ = Op($2, 2, $1        , $3);    }
 | expr vOR expr                                  { $$ = Op($2, 2, $1        , $3);    }
 ;

expr_string:
   vCstSTRING                                     { $$ = sConst($1);           }
 | vVarSTRING                                     { $$ = sVar($1);             }
 | vCHR '(' expr ')'                              { $$ = Op($1, 2, $3, NULL);  } 
 | vLEFT '(' expr_string ',' expr ')'             { $$ = Op($1, 2, $3, $5);    }
 | vMID '(' expr_string ',' expr ',' expr ')'     { $$ = Op($1, 3, $3, $5, $7);}
 | vRIGHT '(' expr_string ',' expr ')'            { $$ = Op($1, 2, $3, $5);    }
 ;

expr:
    vCstFLOAT                                     { $$ = fConst($1);          }
  | vCstINTEGER                                   { $$ = iConst($1);          }
  | vVarFLOAT                                     { $$ = fVar($1);            }
  | vVarINTEGER                                   { $$ = iVar($1);            }
  | vHEXA                                         { $$ = hConst($1);          }
  | function                                      { $$ = $1;                  }
  | expr vPLUS expr                               { $$ = Op($2 , 2 , $1, $3); }
  | expr vMINUS expr                              { $$ = Op($2 , 2 , $1, $3); }
  | expr vMULTIPLY expr                           { $$ = Op($2 , 2 , $1, $3); }
  | expr vDIVIDE expr                             { $$ = Op($2 , 2 , $1, $3); }
  | expr vPOWER expr                              { $$ = Op($2 , 2 , $1, $3); }
  | expr vEQ expr                                 { $$ = Op($2 , 2 , $1, $3); }
  | expr vNE expr                                 { $$ = Op($2 , 2 , $1, $3); }
  | expr vLE expr                                 { $$ = Op($2 , 2 , $1, $3); }
  | expr vLT expr                                 { $$ = Op($2 , 2 , $1, $3); }
  | expr vGE expr                                 { $$ = Op($2 , 2 , $1, $3); }
  | expr vGT expr                                 { $$ = Op($2 , 2 , $1, $3); }
  | '(' expr ')'                                  { $$ = $2;                  }
 ;

cload_expr:
   vCLOAD vCstSTRING                     { $$ = Op($1, 2, sConst($2), NULL);                               }                                                
 | vCLOAD vCstSTRING ',' 'S'             { $$ = Op($1, 2, sConst($2), iConst(K7_SLOW));                    }
 | vCLOAD vCstSTRING ',' 'J'             { $$ = Op($1, 2, sConst($2), iConst(K7_JOIN)  , NULL);            }
 | vCLOAD vCstSTRING ',' 'J' ',' 'S'     { $$ = Op($1, 2, sConst($2), iConst(K7_JOIN)  , iConst(K7_SLOW)); }
 | vCLOAD vCstSTRING ',' 'V'             { $$ = Op($1, 2, sConst($2), iConst(K7_VERIFY), NULL);            }
 | vCLOAD vCstSTRING ',' 'V' ',' 'S'     { $$ = Op($1, 2, sConst($2), iConst(K7_VERIFY), iConst(K7_SLOW)); }
;

csave_expr:
   vCSAVE vCstSTRING                                                           { $$ = Op($1, 2, sConst($2), NULL);                                                                                       }         
 | vCSAVE vCstSTRING ',' vAUTO                                                 { $$ = Op($1, 2, sConst($2), iConst(K7_AUTO));                                                                            }          
 | vCSAVE vCstSTRING ',' 'S'                                                   { $$ = Op($1, 2, sConst($2), iConst(K7_SLOW));                                                                            }
 | vCSAVE vCstSTRING ',' 'S' ',' vAUTO                                         { $$ = Op($1, 3, sConst($2), iConst(K7_SLOW) , iConst(K7_AUTO));                                                          }
 | vCSAVE vCstSTRING ',' 'A' vCstINTEGER ',' 'E' vCstINTEGER                   { $$ = Op($1, 5, sConst($2), iConst(K7_BEGIN), iConst($5), iConst(K7_END), iConst($8));                                   } 
 | vCSAVE vCstSTRING ',' 'A' vCstINTEGER ',' 'E' vCstINTEGER ',' vAUTO         { $$ = Op($1, 6, sConst($2), iConst(K7_BEGIN), iConst($5), iConst(K7_END), iConst($8), iConst(K7_AUTO));                  }
 | vCSAVE vCstSTRING ',' 'A' vCstINTEGER ',' 'E' vCstINTEGER ',' 'S'           { $$ = Op($1, 6, sConst($2), iConst(K7_BEGIN), iConst($5), iConst(K7_END), iConst($8), iConst(K7_SLOW));                  }
 | vCSAVE vCstSTRING ',' 'A' vCstINTEGER ',' 'E' vCstINTEGER ',' 'S' ',' vAUTO { $$ = Op($1, 6, sConst($2), iConst(K7_BEGIN), iConst($5), iConst(K7_END), iConst($8), iConst(K7_SLOW), iConst(K7_AUTO)); }
;

data_expr:
    expr                                          { $$ = $1;                   }
  | expr ',' data_expr                            { $$ = Op(LISTE, 2, $1, $3); }
  ;                                               
                                                  
def_expr:                                         
    vFN vVarFLOAT '(' expr ')' vEQ expr           { $$ = Op(FN , 3, fVar($2), $4, $7); }
  | vUSR vEQ expr                                 { $$ = Op(USR, 2, $3      , NULL);   }
  ;

dim_expr:
    vDIM vVarFLOAT  '(' dim_2_expr ')'            { $$ = Op($1, 2, fVar($2), $4); }
  | vDIM vVarINTEGER '(' dim_2_expr ')'           { $$ = Op($1, 2, iVar($2), $4); }
  | vDIM vVarSTRING '(' dim_2_expr ')'            { $$ = Op($1, 2, sVar($2), $4); }
  ;                                               
                                                  
dim_2_expr:                                       
    vCstINTEGER                                   { $$ = iConst($1);                   }
  | vCstINTEGER ',' dim_2_expr                    { $$ = Op(LISTE, 2, iConst($1), $3); }
  ;                                               
                                                  
for_expr:                                         
    vFOR vVarFLOAT vEQ expr vTO expr              { $$ = Op($1, 3, fVar($2), $4, $6);     }
  | vFOR vVarFLOAT vEQ expr vTO expr vSTEP expr   { $$ = Op($1, 4, fVar($2), $4, $6, $8); }
  | vFOR vVarINTEGER vEQ expr vTO expr            { $$ = Op($1, 3, iVar($2), $4, $6);     }
  | vFOR vVarINTEGER vEQ expr vTO expr vSTEP expr { $$ = Op($1, 5, iVar($2), $4, $6, $8); }
  ;

goto_expr:
    vCstINTEGER { $$ = iConst($1); }
  ;

gosub_expr:
    vCstINTEGER { $$ = iConst($1); }
  ;

if_expr:
    vIF expr vTHEN instructions %prec IFX         { $$ = Op($1, 2 , $2, $4);                     }
  | vIF expr vTHEN vCstINTEGER %prec IFX          { $$ = Op($1, 2 , $2, iConst($4));             }
  | vIF expr vTHEN vCstINTEGER vELSE instructions { $$ = Op($1, 3 , $2, iConst($4), $6);         }
  | vIF expr vTHEN vCstINTEGER vELSE vCstINTEGER  { $$ = Op($1, 3 , $2, iConst($4), iConst($6)); }
  | vIF expr vTHEN command vELSE instructions     { $$ = Op($1, 3 , $2, $4, $6);                 }
 ;

input_expr:
    vVarFLOAT                                     { $$ = fVar($1); }
  | vVarINTEGER                                   { $$ = iVar($1); }
  | vVarSTRING                                    { $$ = sVar($1); }
  | vVarFLOAT ',' input_expr                      { $$ = Op(LISTE, 2, iVar($1), $3); }
  | vVarINTEGER ',' input_expr                    { $$ = Op(LISTE, 2, fVar($1), $3); }
  | vVarSTRING ',' input_expr                     { $$ = Op(LISTE, 2, sVar($1), $3); }
  ;                                               
                                                  
print_expr:                                       
                                                  { $$ = Op(NOP, 2, NULL, NULL);   }
  | vAT expr ',' expr ';' print_2_expr            { $$ = Op(AT, 3, $2  , $4, $6);  }
  | print_2_expr                                  { $$ = $1;                       }
  ;                                               
                                                  
print_2_expr:                                     
    expr                                          { $$ = $1;                   }
  | expr ',' print_2_expr                         { $$ = Op(LISTE, 3, $1, $3); }
  | expr ';' print_2_expr                         { $$ = Op(LISTE, 3, $1, $3); }
  ;                                               
                                                  
read_expr:                                        
    vVarFLOAT                                     { $$ = fVar($1);                   }
  | vVarINTEGER                                   { $$ = iVar($1);                   }
  | vVarSTRING                                    { $$ = sVar($1);                   }
  | vVarFLOAT  ',' read_expr                      { $$ = Op(LISTE, 2, fVar($1), $3); }
  | vVarINTEGER ',' read_expr                     { $$ = Op(LISTE, 2, iVar($1), $3); }
  | vVarSTRING ',' read_expr                      { $$ = Op(LISTE, 2, sVar($1), $3); }
  ;                                               
                                                  
recall_expr:                                      
   vRECALL vVarINTEGER ',' vCstSTRING             { $$ = Op($1, 2, iVar($2), sConst($4));                  }
 | vRECALL vVarINTEGER ',' vCstSTRING ',' 'S'     { $$ = Op($1, 3, iVar($2), sConst($4), iConst(K7_SLOW)); }
 | vRECALL vVarFLOAT ',' vCstSTRING               { $$ = Op($1, 2, fVar($2), sConst($4));                  }
 | vRECALL vVarFLOAT ',' vCstSTRING ',' 'S'       { $$ = Op($1, 3, fVar($2), sConst($4), iConst(K7_SLOW)); }
 | vRECALL vVarSTRING ',' vCstSTRING              { $$ = Op($1, 2, sVar($2), sConst($4));                  }
 | vRECALL vVarSTRING ',' vCstSTRING ',' 'S'      { $$ = Op($1, 3, sVar($2), sConst($4), iConst(K7_SLOW)); }
;

store_expr:
   vSTORE vVarINTEGER ',' vCstSTRING              { $$ = Op($1, 2, iVar($2), sConst($4));                  }
 | vSTORE vVarINTEGER ',' vCstSTRING ',' 'S'      { $$ = Op($1, 3, iVar($2), sConst($4), iConst(K7_SLOW)); }
 | vSTORE vVarFLOAT ',' vCstSTRING                { $$ = Op($1, 2, fVar($2), sConst($4));                  }
 | vSTORE vVarFLOAT ',' vCstSTRING ',' 'S'        { $$ = Op($1, 3, fVar($2), sConst($4), iConst(K7_SLOW)); }
 | vSTORE vVarSTRING ',' vCstSTRING               { $$ = Op($1, 2, sVar($2), sConst($4));                  }
 | vSTORE vVarSTRING ',' vCstSTRING ',' 'S'       { $$ = Op($1, 3, sVar($2), sConst($4), iConst(K7_SLOW)); }
;

%%

/*================================*/
/*=== DEFINITION DES VARIABLES ===*/
/*================================*/
       int    NumLineBasic;   /* Numero de la ligne basic traitee */
extern int    yylineno;
extern char * yytext;
extern int    yydebug;        /* Pour le debuggage du lexer       */
extern FILE * yyin;

extern struct nodetypeliste  * pPRG         = NULL;
extern struct nodetypeliste  * pListeLines  = NULL;

extern NodeVariable  pListeVarInt    = NULL;
extern NodeVariable  pListeVarFlt    = NULL;
extern NodeVariable  pListeVarStr    = NULL;
extern NodeVariable  ptmpListeVarInt = NULL;
extern NodeVariable  ptmpListeVarFlt = NULL;
extern NodeVariable  ptmpListeVarStr = NULL;



/*================================*/
/*=== DEFINITION DES FONCTIONS ===*/
/*================================*/
int yyerror(char *s) { fprintf(stderr, "parser: line %d: <%s>, <%s>\n", yylineno, s, &yytext); return 0; }


/*=====================*/
/*=== FONCTION MAIN ===*/
/*=====================*/

int main(int argc, char * * argv) {
FILE * file;

/* Mode debug */
/* yydebug = 1; */


/* Initialisation des variables */
NumLineBasic = 0;

/* Ouverture du fichier en entree si necessaire */
if (argc > 1) {
   file = fopen(argv[1], "r");
   if (!file) {
      fprintf(stderr, "parser: Could not open '%s'\n", argv[1]);
      exit(1);
   }
   yyin = file;
}

/* L'appel du parser */
yyparse();
 
/* Quelques affichages */ 
ptmpListeVarInt = pListeVarInt; 
ptmpListeVarFlt = pListeVarFlt; 
ptmpListeVarStr = pListeVarStr; 
 
printf("Liste de variables de type entier:\n"); (void) printVariable(ptmpListeVarInt);                                                                                   
printf("Liste de variables de type float :\n"); (void) printVariable(ptmpListeVarFlt);
printf("Liste de variables de type string:\n"); (void) printVariable(ptmpListeVarStr);

printf("Nb de variables de type entier: '%d'\n", countVariable(ptmpListeVarInt));                                                                                   
printf("Nb de variables de type float : '%d'\n", countVariable(ptmpListeVarFlt));
printf("Nb de variables de type string: '%d'\n", countVariable(ptmpListeVarStr));


return 0;
}
