#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include "parse.h"
#include "y.tab.h"
#include "ex1.h"

extern struct keyword token[MAX_TOKEN];

char * tabTYPE_OP[]= { "fConst", "iConst", "sConst", "tHexa", "fVar", "iVar", "sVar", "Op" };

/*=== Add a variable into the list of variables ===*/
/*=== return : 0 : OK                           ===*/
/*===          1 : ALREADY EXISTS               ===*/
/*===          2 : ERROR                        ===*/
/*===          3 : ERROR  NAME IS NULL OR ""    ===*/
int addVariable(char * name, NodeVariable * pListeVariable)
{
  char sTmpName[2 + 1] = "";
  int  iCr = 0;

  if (name != NULL)
  {
    if (!strcmp(name, "")) return 3;

    strncpy(sTmpName, name, 2);
    sTmpName[2] = '\0';

    if (strlen(name) > 2 ) { printf("Liste.c: addVariable(): La variable '%s' sera renomme en '%s'\n", name, sTmpName); }

    iCr = seekVariable(sTmpName, *pListeVariable);
    if (iCr == 0) *pListeVariable = createVariable(sTmpName, *pListeVariable);

    return iCr;
   }

  return 3;
}

/*=== Create a new variable into the list of variables ===*/
/*=== return : 0 OK                                    ===*/
/*===          2 ERROR                                 ===*/
NodeVariable createVariable(char * sName, NodeVariable pBegin)
{
  struct nodevariable * pNew;
  NodeVariable p = NULL, q = NULL;    

  pNew = (struct nodevariable *) malloc(sizeof(struct nodevariable));

  if (pNew == NULL)  { printf("Liste.c: createVariable(): Error alloc memory\n"); return 2; }

  pNew->sName = (char *) strdup(sName);
  
  if (pBegin == NULL) {  	
        pBegin       = pNew;
        pBegin->Next = NULL;
  } 
  else if (strcmp(sName, pBegin->sName) <= 0)
  {                                                                                                               
  	pNew->Next = pBegin;
  	pBegin     = pNew;
  }
  else
  {
  	p = pBegin;
  	q = pBegin->Next;
  	while ( (q != NULL) && (strcmp(sName, q->sName) > 0) )
  	{  	  
  	  p = q;
  	  q = q->Next;
  	}
  	p->Next    = pNew;
  	pNew->Next = q;
  }

  return pBegin;
}

/*=== Seek a variable into the list of variables ===*/
/*=== return : 0 not found                       ===*/
/*===          1 found                           ===*/
int seekVariable(char * sName, NodeVariable pListeVariable)
{
 NodeVariable pTmp;
 int iCr;

 pTmp = pListeVariable;

 while (pTmp != NULL)
 {
   if (!strcmp(pTmp->sName, sName)) return 1;
   pTmp = pTmp->Next;
  }

 return 0;
}

/*=== Print a  list of variables ===*/
/*=== return : none              ===*/
void printVariable(NodeVariable pListeVariable)
{
 NodeVariable pTmp;
 int i = 0;

 /* printf("Liste.c: printVariable(): Debut\n"); */

 pTmp = pListeVariable;
 i = 0;
 while (pTmp != NULL)
 {
   printf("i=%i, name='%s'\n", i , pTmp->sName);
   pTmp = pTmp->Next;
   i++;
  }

}

/*=== Count number variable from a list of variables ===*/
/*=== return : the number                            ===*/
int countVariable(NodeVariable pListeVariable)
{  
 NodeVariable pTmp = NULL;
 int i = 0; 

 pTmp = pListeVariable;
 i = 0;
 
 while (pTmp != NULL)
 {   
   pTmp = pTmp->Next;
   i++;
  }
 return i;
}

/*=== Seek the position of variable from a list of variables ===*/
/*=== return : the position                                  ===*/
int posVariable(char * sName, NodeVariable pListeVariable)
{  
 NodeVariable pTmp = NULL;
 int i = 0; 

 pTmp = pListeVariable;
 i = 0;
 
 /* printf("liste.c: posVariable(), <%s>\n", sName); */
 
 while ( (pTmp != NULL) && (strcmp(sName, pTmp->sName) != 0) )
 {      
   pTmp = pTmp->Next;
   i++;
  }
 return i;
}



/*=== CONSTANT INTEGER ===*/
nodeType * iConst(int value) {
 nodeType * p;

 if ((p = malloc(sizeof(ConstNodeType))) == NULL) yyerror("liste.c : iConst(): out of memory");

 p->type         = tiConst;
 p->con.u.iValue = value;

 return p;
}


/*=== CONSTANT FLOAT ===*/
nodeType * fConst(float value) {
 nodeType * p;

 if ((p = malloc(sizeof(ConstNodeType))) == NULL) yyerror("liste.c : fConst(): out of memory");

 p->type         = tfConst;
 p->con.u.fValue = value;

 return p;
}

/*=== CONSTANT HEXADECIMAL ===*/
nodeType * hConst(char * sValue) {
 nodeType * p;
 int i;

 if ((p = malloc(sizeof(HexaNodeType))) == NULL) yyerror("liste.c : hConst(): out of memory");

 p->type = tHexa;
 sscanf(sValue, "#%x", &(p->hex).iValue);
 printf("hConst: sValue=%s, iValue=%d\n", sValue, p->hex.iValue);

 return p;
}

/*=== CONSTANT STRING ===*/
nodeType * sConst(char * value) {
 nodeType * p;


 if ((p = malloc(sizeof(ConstNodeType))) == NULL) yyerror("liste.c : sConst(): out of memory");

 p->type         = tsConst;
 p->con.u.sValue = (char *) strdup(value);

 return p;
}

/*=== VARIABLE FLOAT ===*/
nodeType * fVar(char * name) {
 nodeType * p;

 /* printf("liste.c: fVar(): Debut\n"); */

 if ((p = malloc(sizeof(ConstNodeType))) == NULL) yyerror("liste.c : fVar(): out of memory");

 p->type = tfVar;
 strncpy(p->var.sValue, name, 3);
 p->var.sValue[3]='\0';

 /* Ajout du nom dans la liste des variable de type float */
 addVariable(name, &pListeVarFlt);

 return p;
}


/*=== VARIABLE INTEGER ===*/
nodeType * iVar(char * name) {
 nodeType * p;

 /* printf("liste.c: iVar(): Debut\n"); */

 if ((p = malloc(sizeof(ConstNodeType))) == NULL) yyerror("liste.c : iVar(): out of memory");

 p->type = tiVar;
 strncpy(p->var.sValue, name, 2);
 p->var.sValue[2]='\0';

 /* Ajout du nom dans la liste des variable de type integer */
 addVariable(name, &pListeVarInt);

 return p;
}

/*=== VARIABLE STRING ===*/
nodeType * sVar(char * name) {
 nodeType * p;

 /* printf("liste.c: sVar(): Debut\n"); */

 if ((p = malloc(sizeof(ConstNodeType))) == NULL) yyerror("liste.c : sVar(): out of memory");

 p->type = tsVar;
 strncpy(p->var.sValue, name, 3);
 p->var.sValue[3]='\0';

 /* Ajout du nom dans la liste des variable de type string */
 addVariable(name, &pListeVarStr);

 return p;
}

/*=== OPERATION ===*/
nodeType * Op(int oper, int nops, ...) {
 nodeType * p = NULL;
 va_list    ap;
 size_t     size;
 int        i = 0;

 size = sizeof(OpNodeType) + (nops - 1) * sizeof(nodeType * );

 if ((p = malloc(size)) == NULL) yyerror("liste.c: op(): out of memory");

 p->type    = tOp;
 p->op.oper = oper;
 p->op.nops = nops;

 printf("liste.c : Op(): oper(%d)='%s', nops=%d\n", oper, oper < 1000 ? token[oper].name : "", nops);

 va_start(ap, nops);
 for (i = 0; i < nops; i++) {
     p->op.args[i] = va_arg(ap, nodeType *);
     if (p->op.args[i] != NULL) printf("liste.c : Op(): Op='%s'\n", tabTYPE_OP[p->op.args[i]->op.type]);
 }
 va_end(ap);
 return p;
}

/*=== LIBERATION DE LA MEMOIRE ALLOUEE ===*/
void freeNode(nodeType * p) {
 int i;

 if (!p) return;
 if (p->type == tOp) for (i = 0; i < p->op.nops; i++) freeNode(p->op.args[i]);
 free(p);
}


/*=== AFFICHAGE DE LA CONSTITUTION D'UN NOEUD ===*/
int print (nodeType * Node) {
 int i;

 if (!Node) { printf(" liste.c : print(): Sortie\n"); return; }

 /* printf("print, =%d, oper=%d\n", Node->type, Node->op.oper); */

 switch (Node->type) {
        case tOp     :
                       /* Traitement special dans le cas d'une ligne */
                       if (Node->op.oper == LIGNE) {
                          printf("L_%d: \n", Node->op.args[0]->con.u.iValue);
                          print(Node->op.args[1]);
                       } else {
                           if (Node->op.oper < 1000) {
                              /* printf("Liste.c: print(): fct(), oper=%d AVANT \n", Node->op.oper); */
                              token[Node->op.oper].fct(Node);
                              /* printf("Liste.c: print(): fct(), oper=%d APRES\n", Node->op.oper); */
                           } else {
                              printf(" |token ='%d'|", Node->op.oper);
                           }
                       }
                       break;
        case tfConst : printf(" print(): fCst=%f\n", Node->con.u.fValue);     break;
        case tiConst : printf(" print(): iCst=%d\n", Node->con.u.iValue);     break;
        case tsConst : printf(" print(): sCst=%s\n", Node->con.u.sValue);     break;
        case tiVar   : printf(" print(): iVar=%s\n", Node->var.sValue);       break;
        case tfVar   : printf(" print(): fVar=%s\n", Node->var.sValue);       break;
        case tsVar   : printf(" print(): sVar=%s\n", Node->var.sValue);       break;
        case tHexa   : printf(" print(): hexa=%d\n", Node->hex.iValue);       break;
        default      : printf("\nprint: Erreur\n");                           break;
 }
 /* printf("liste.c: print() : FIN\n"); */
}

/*=== Add a line into the list of line          ===*/
/*=== return : 0 : OK                           ===*/
/*===          1 : Error                        ===*/
int addLine(nodeType * sLine, struct nodetypeliste * * pListeLine)
{
  nodeTypeListe * pTmpType = NULL;

  pTmpType = (nodeTypeListe *) malloc(sizeof(nodeTypeListe));

  if (pTmpType == NULL) { printf("liste.c: addLine(): pTmpType NULL\n"); return 1; }

  pTmpType->Node = sLine;

  if (*pListeLine == NULL) {
         pTmpType->Next = NULL;
         pTmpType->Prev = NULL;
        *pListeLine     = pTmpType;
         pPRG           = *pListeLine;
  } else {
        pTmpType->Next      = (*pListeLine)->Next;
        (*pListeLine)->Next = pTmpType;
        pTmpType->Prev      = *pListeLine;
        *pListeLine         = pTmpType;
  }

  return 0;
}


/*=== Print the program ===*/
/*=========================*/
int printPRG(struct nodetypeliste * pListeLine) {
 struct nodetypeliste * pTmpListLine = NULL;

 pTmpListLine = pListeLine;

 while (pTmpListLine != NULL)
 {
   print(pTmpListLine->Node);
   pTmpListLine = pTmpListLine->Prev;
  }
}


/*=== COMPILE THE CODE ===*/
/*========================*/
int COMPILATION(struct nodetypeliste * pListeLine) {
struct nodetypeliste * pTmpListLine = NULL;
int iNumLine = 0;

 pTmpListLine = pListeLine;
 iLib1 = 0;
 iLib2 = 0;

 if (pListeLine == NULL) printf("liste.c :COMPILATION(): pListeLine is a NULL pointer\n");

 printf("\n\t .org %s\n", Int2StrHexa(0x500)); 
 printf("\n\t v0ACC1    equ 0x00D0\n");
 printf("\n\t v0ACC2    equ 0x00D8\n");
 printf("\n\t v0STRING1 equ 0x00DE\n"); 
  
 while (pTmpListLine != NULL) {
   print(pTmpListLine->Node);
   pTmpListLine = pTmpListLine->Next;
 }
  
 printf("\n\t.end\n");
}
